# This file was auto-generated by Fern from our API Definition.

import typing

BorderStyle = typing.Union[
    typing.Literal["dotted", "dashed", "solid", "solid_medium", "solid_thick", "double"], typing.Any
]
