# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.run_task_response import RunTaskResponse
from .types.run_task_request_task_type import RunTaskRequestTaskType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTasksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def run_task(
        self,
        *,
        task_id: str,
        arguments: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        task_type: typing.Optional[RunTaskRequestTaskType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RunTaskResponse]:
        """
        Run a [task](https://resources.athenaintel.com/docs/task-studio/home) and wait for the result.

        Executes a serverless function script or flow synchronously. Server handles polling internally.

        Parameters
        ----------
        task_id : str
            The unique identifier (path) of the task. Example: 'f/public/databricks_describe_table'

        arguments : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arguments to pass to the task

        task_type : typing.Optional[RunTaskRequestTaskType]
            Type: 'script' or 'flow'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RunTaskResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v0/tools/tasks/run",
            method="POST",
            json={
                "arguments": arguments,
                "task_id": task_id,
                "task_type": task_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RunTaskResponse,
                    parse_obj_as(
                        type_=RunTaskResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTasksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def run_task(
        self,
        *,
        task_id: str,
        arguments: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        task_type: typing.Optional[RunTaskRequestTaskType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RunTaskResponse]:
        """
        Run a [task](https://resources.athenaintel.com/docs/task-studio/home) and wait for the result.

        Executes a serverless function script or flow synchronously. Server handles polling internally.

        Parameters
        ----------
        task_id : str
            The unique identifier (path) of the task. Example: 'f/public/databricks_describe_table'

        arguments : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Arguments to pass to the task

        task_type : typing.Optional[RunTaskRequestTaskType]
            Type: 'script' or 'flow'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RunTaskResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v0/tools/tasks/run",
            method="POST",
            json={
                "arguments": arguments,
                "task_id": task_id,
                "task_type": task_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RunTaskResponse,
                    parse_obj_as(
                        type_=RunTaskResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
